{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Create Network Association" %}{% endblock %}

{% block main %}
  {% include "project/bgpvpn/_create_network_association.html" %}
{% endblock %}
